searchNodes=[{"doc":"Helpers for defining clustered test cases. Use this in place of ExUnit.Case when defining test modules where you will be defining clustered tests. Elixir.ExUnit.ClusteredCase extends ExUnit.Case to provide additional helpers for those tests. Since Elixir.ExUnit.ClusteredCase is an extension of ExUnit.Case , it takes the same options, and imports the same test helpers and callbacks. It adds new helpers, scenario/3 , node_setup/1 , and node_setup/2 , and aliases the Elixir.ExUnit.ClusteredCase.Cluster module for convenience. Examples defmodule KVStoreTests do # Use the module use ExUnit.ClusteredCase # Define a clustered scenario scenario &quot;given a healthy cluster&quot; , [ cluster_size : 3 ] do # Set up each node in the cluster prior to each test node_setup do { :ok , _ } = Application . ensure_all_started ( :kv_store ) end # Define a test to run in this scenario test &quot;always pass&quot; do assert true end end end Context All tests receive a context as an argument, just like with ExUnit.Case , the primary difference to be aware of is that the context contains a key, :cluster , which is the pid of the cluster manager, and is used to invoke functions in the Elixir.ExUnit.ClusteredCase.Cluster module during tests. defmodule KVStoreTests do use ExUnit.ClusteredCase scenario &quot;given a healthy cluster&quot; , [ cluster_size : 3 ] do # You can use normal setup functions to setup context for the # test, this is run once prior to each test setup do { :ok , foo : :bar } end # Like `setup`, but is run on all nodes prior to each test node_setup do { :ok , _ } = Application . ensure_all_started ( :kv_store ) end test &quot;cluster has three nodes&quot; , %{ cluster : c } = context do assert length ( Cluster . members ( c ) ) == 3 end end end See the ExUnit.Case documentation for information on tags, filters, and more.","ref":"ExUnit.ClusteredCase.html","title":"ExUnit.ClusteredCase","type":"module"},{"doc":"Like ExUnit.Callbacks.setup/1 , but is executed on every node in the cluster. You can pass a block, a unary function as an atom, or a list of such atoms. If you pass a unary function, it receives the test setup context, however unlike setup/1 , the value returned from this function does not modify the context. Use setup/1 or setup/2 for that. NOTE: This callback is invoked on each node in the cluster for the given scenario. Examples def start_apps ( _context ) do { :ok , _ } = Application . ensure_all_started ( :kv_store ) :ok end scenario &quot;given a healthy cluster&quot; , [ cluster_size : 3 ] do node_setup :start_apps node_setup do # This form is also acceptable { :ok , _ } = Application . ensure_all_started ( :kv_store ) end end","ref":"ExUnit.ClusteredCase.html#node_setup/1","title":"ExUnit.ClusteredCase.node_setup/1","type":"macro"},{"doc":"Same as node_setup/1 , but receives the test setup context as a parameter. Examples scenario &quot;given a healthy cluster&quot; , [ cluster_size : 3 ] do node_setup _context do # Do something on each node end end","ref":"ExUnit.ClusteredCase.html#node_setup/2","title":"ExUnit.ClusteredCase.node_setup/2","type":"macro"},{"doc":"Creates a new clustered test scenario. Usage of this macro is similar to that of ExUnit.Case.describe/2 , but has some differences. While describe/2 simply groups tests under a common description, scenario/3 both describes the group of tests, and initializes a cluster which will be made available for each test in that scenario. NOTE: It is important to be aware that each scenario is a distinct cluster, and that all tests within a single scenario are running against the same cluster. If tests within a scenario may conflict with one another - perhaps by modifying shared state, or triggering crashes which may bring down shared processes, etc., then you have a couple options: Disable async testing for the module Modify your tests to prevent conflict, e.g. writing to different keys in a k/v store, rather than the same key Split the scenario into many, where the tests can run in isolation. Options You can configure a scenario with the following options: cluster_size: integer , will create a cluster of the given size, this option is mutually exclusive with :nodes , if the latter is used, this option will be ignored. nodes: [[node_opt]] , a list of node specifications to use when creating the cluster, see Elixir.ExUnit.ClusteredCase.Node.node_opt/0 for specific options available. If used, :cluster_size is ignored. env: [{String.t, String.t}] , will set the given key/values in the environment when creating nodes. If you need different values for each node, you will need to use :nodes erl_flags: [String.t] , additional arguments to pass to erl when creating nodes, like :env , if you need different args for each node, you will need to use :nodes config: Keyword.t , configuration overrides to apply to all nodes in the cluster boot_timeout: integer , the amount of time to allow for nodes to boot, in milliseconds init_timeout: integer , the amount of time to allow for nodes to be initialized, in milliseconds Examples defmodule KVStoreTest do use ExUnit.ClusteredCase @scenario_opts [ cluster_size : 3 ] scenario &quot;given a healthy cluster&quot; , @scenario_opts do node_setup do { :ok , _ } = Application . ensure_all_started ( :kv_store ) end test &quot;writes are replicated to all nodes&quot; , %{ cluster : cluster } do writer = Cluster . random_member ( cluster ) key = self ( ) value = key assert Cluster . call ( writer , KVStore , :put , [ key , value ] ) == :ok results = Cluster . map ( cluster , KVStore , :get , [ key ] ) assert Enum . all? ( results , fn val -&gt; val == value end ) end end end Since all scenarios are also describes, you can run all the tests for a scenario by it's description: mix test -- only describe :&quot;given a healthy cluster&quot; or by passing the exact line the scenario starts on: mix test path / to / file : 123 Like describe/2 , you cannot nest scenario/3 . Use the same technique of named setups recommended in the describe/2 documentation for composition.","ref":"ExUnit.ClusteredCase.html#scenario/3","title":"ExUnit.ClusteredCase.scenario/3","type":"macro"},{"doc":"This module is responsible for managing the setup and lifecycle of a single cluster.","ref":"ExUnit.ClusteredCase.Cluster.html","title":"ExUnit.ClusteredCase.Cluster","type":"module"},{"doc":"Invoke a function on a specific member of the cluster","ref":"ExUnit.ClusteredCase.Cluster.html#call/2","title":"ExUnit.ClusteredCase.Cluster.call/2","type":"function"},{"doc":"Invoke a function on a specific member of the cluster","ref":"ExUnit.ClusteredCase.Cluster.html#call/4","title":"ExUnit.ClusteredCase.Cluster.call/4","type":"function"},{"doc":"Applies a function on all nodes in the cluster.","ref":"ExUnit.ClusteredCase.Cluster.html#each/2","title":"ExUnit.ClusteredCase.Cluster.each/2","type":"function"},{"doc":"Applies a function on all nodes in the cluster.","ref":"ExUnit.ClusteredCase.Cluster.html#each/4","title":"ExUnit.ClusteredCase.Cluster.each/4","type":"function"},{"doc":"Heals all partitions in the cluster.","ref":"ExUnit.ClusteredCase.Cluster.html#heal/1","title":"ExUnit.ClusteredCase.Cluster.heal/1","type":"function"},{"doc":"Kills a running node in a cluster. Expects the cluster and node to stop.","ref":"ExUnit.ClusteredCase.Cluster.html#kill_node/2","title":"ExUnit.ClusteredCase.Cluster.kill_node/2","type":"function"},{"doc":"Get the captured log for a specific node in the cluster","ref":"ExUnit.ClusteredCase.Cluster.html#log/1","title":"ExUnit.ClusteredCase.Cluster.log/1","type":"function"},{"doc":"Maps a function across all nodes in the cluster. Returns a list of results, where each element is the result from one node.","ref":"ExUnit.ClusteredCase.Cluster.html#map/2","title":"ExUnit.ClusteredCase.Cluster.map/2","type":"function"},{"doc":"Maps a function across all nodes in the cluster. Returns a list of results, where each element is the result from one node.","ref":"ExUnit.ClusteredCase.Cluster.html#map/4","title":"ExUnit.ClusteredCase.Cluster.map/4","type":"function"},{"doc":"Retrieve a list of nodes in the given cluster","ref":"ExUnit.ClusteredCase.Cluster.html#members/1","title":"ExUnit.ClusteredCase.Cluster.members/1","type":"function"},{"doc":"Partition the cluster based on the provided specification. You can specify partitions in one of the following ways: As an integer representing the number of partitions As a list of integers representing the number of nodes in each partition As a list of lists, where each sub-list contains the nodes in that partition If your partitioning specification cannot be complied with, an error is returned Examples test &quot;partition by number of partitions&quot; , %{ cluster : c } do Cluster . partition ( c , 2 ) end test &quot;partition by number of nodes per partition&quot; , %{ cluster : c } do Cluster . partition ( c , [ 2 , 2 ] ) end test &quot;partition by list of nodes in each partition&quot; , %{ cluster : c } do Cluster . partition ( c , [ [ :a , :b ] , [ :c , :d ] ] ) end","ref":"ExUnit.ClusteredCase.Cluster.html#partition/2","title":"ExUnit.ClusteredCase.Cluster.partition/2","type":"function"},{"doc":"Retrieve the partitions this cluster is composed of","ref":"ExUnit.ClusteredCase.Cluster.html#partitions/1","title":"ExUnit.ClusteredCase.Cluster.partitions/1","type":"function"},{"doc":"Retrieve the name of a random node in the given cluster","ref":"ExUnit.ClusteredCase.Cluster.html#random_member/1","title":"ExUnit.ClusteredCase.Cluster.random_member/1","type":"function"},{"doc":"Repartitions the cluster based on the provided specification. See partition/2 for specification details. Repartitioning performs the minimal set of changes required to converge on the partitioning scheme in an attempt to minimize the amount of churn. That said, some churn is expected, so bear that in mind when writing tests with partitioning events involved.","ref":"ExUnit.ClusteredCase.Cluster.html#repartition/2","title":"ExUnit.ClusteredCase.Cluster.repartition/2","type":"function"},{"doc":"Starts a new cluster with the given specification","ref":"ExUnit.ClusteredCase.Cluster.html#start/1","title":"ExUnit.ClusteredCase.Cluster.start/1","type":"function"},{"doc":"Stops a running cluster. Expects the pid of the cluster manager process.","ref":"ExUnit.ClusteredCase.Cluster.html#stop/1","title":"ExUnit.ClusteredCase.Cluster.stop/1","type":"function"},{"doc":"Stops a running node in a cluster. Expects the cluster and node to stop.","ref":"ExUnit.ClusteredCase.Cluster.html#stop_node/2","title":"ExUnit.ClusteredCase.Cluster.stop_node/2","type":"function"},{"doc":"","ref":"ExUnit.ClusteredCase.Cluster.html#t:callback/0","title":"ExUnit.ClusteredCase.Cluster.callback/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Cluster.html#t:cluster_opt/0","title":"ExUnit.ClusteredCase.Cluster.cluster_opt/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Cluster.html#t:cluster_opts/0","title":"ExUnit.ClusteredCase.Cluster.cluster_opts/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Cluster.html#t:node_spec/0","title":"ExUnit.ClusteredCase.Cluster.node_spec/0","type":"type"},{"doc":"This module handles starting new nodes You can specify various options when starting a node: :name , takes either a string or an atom, in either long or short form, this sets the node name and id. :boot_timeout , specifies the amount of time to wait for a node to perform it's initial boot sequence before timing out. :init_timeout , specifies the amount of time to wait for the node agent to complete initializing the node (loading and starting required applications, applying configuration, loading test modules, and executing post-start functions). :erl_flags , a list of arguments to pass to erl when starting the node, e.g. [&quot;-init_debug&quot;] . :env , a list of tuples containing environment variables to export in the node's environment, e.g. [{&quot;PORT&quot;, &quot;8080&quot;}] . :config , a Keyword list containing configuration overrides to apply to the node, should be in the form of [app: [key: value]] . :post_start_functions , a list of functions, either captured or in {module, function, args} format, which will be invoked on the node after it is booted and initialized. Functions must be zero-arity. :stdout , redirect output to a device or process with stdout: :standard_error | :standard_io | pid . :capture_log , capture the entire log from a node with capture_log: true , can get the captured logs for a specific node with Cluster.log(node) .","ref":"ExUnit.ClusteredCase.Node.html","title":"ExUnit.ClusteredCase.Node","type":"module"},{"doc":"","ref":"ExUnit.ClusteredCase.Node.html#t:fun/0","title":"ExUnit.ClusteredCase.Node.fun/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Node.html#t:node_error/0","title":"ExUnit.ClusteredCase.Node.node_error/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Node.html#t:node_opt/0","title":"ExUnit.ClusteredCase.Node.node_opt/0","type":"type"},{"doc":"","ref":"ExUnit.ClusteredCase.Node.html#t:node_opts/0","title":"ExUnit.ClusteredCase.Node.node_opts/0","type":"type"}]